import java.io.IOException;
import java.text.ParseException;

import fr.cnes.dynvol.psimu.calc.integrator.PsimuPropagationData;
import fr.cnes.dynvol.psimu.calc.integrator.firstOrderIntegrators.RK4data;
import fr.cnes.dynvol.psimu.calc.main.Psimu;
import fr.cnes.dynvol.psimu.calc.main.UserFramesConfsEnum;
import fr.cnes.dynvol.psimu.exception.PsimuException;
import fr.cnes.dynvol.psimu.exception.PsimuPropagationException;
import fr.cnes.genopus.nonWidget.exception.NonWidgetException;
import fr.cnes.sirius.addons.patriusdataset.PatriusDataset;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.Vehicle;
import fr.cnes.sirius.patrius.assembly.models.MassModel;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.vehicle.VehicleSurfaceModel;
import fr.cnes.sirius.patrius.attitudes.AttitudeLaw;
import fr.cnes.sirius.patrius.attitudes.LofOffset;
import fr.cnes.sirius.patrius.bodies.ExtendedOneAxisEllipsoid;
import fr.cnes.sirius.patrius.forces.ForceModel;
import fr.cnes.sirius.patrius.forces.ForceModelsData;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory.GravityFieldNames;
import fr.cnes.sirius.patrius.frames.Frame;
import fr.cnes.sirius.patrius.frames.FramesFactory;
import fr.cnes.sirius.patrius.frames.LOFType;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Sphere;
import fr.cnes.sirius.patrius.math.util.FastMath;
import fr.cnes.sirius.patrius.orbits.KeplerianOrbit;
import fr.cnes.sirius.patrius.orbits.PositionAngle;
import fr.cnes.sirius.patrius.propagation.MassProvider;
import fr.cnes.sirius.patrius.propagation.SpacecraftState;
import fr.cnes.sirius.patrius.time.AbsoluteDate;
import fr.cnes.sirius.patrius.time.TimeScale;
import fr.cnes.sirius.patrius.time.TimeScalesFactory;
import fr.cnes.sirius.patrius.utils.Constants;
import fr.cnes.sirius.patrius.utils.exception.PatriusException;

/**
 * Basic tutorial to use PSIMU with a specific frames configuration.
 * @author goesterjf
 *
 */
public class PsimuBasicWithFramesConfig {

    public static void main(String[] args)
            throws PatriusException, NonWidgetException, IOException, ParseException, PsimuException, PsimuPropagationException {
        
        // Patrius Dataset initialization
        PatriusDataset.addResourcesFromPatriusDataset() ;

        // Recovery of the UTC time scale using a "factory"
        final TimeScale TUC = TimeScalesFactory.getUTC();
        
        // Reference frames
        final Frame GCRF = FramesFactory.getGCRF();
        final Frame ITRF = FramesFactory.getITRF();

        // Constants that will be used later
        final double MU   = Constants.WGS84_EARTH_MU;
        final double REQ  = Constants.WGS84_EARTH_EQUATORIAL_RADIUS;
        final double FLAT = Constants.WGS84_EARTH_FLATTENING;
        final ExtendedOneAxisEllipsoid EARTH = new ExtendedOneAxisEllipsoid(REQ, FLAT, ITRF, "EARTH");
        
        // ORBIT
        
        final AbsoluteDate date = new AbsoluteDate("2010-01-01T12:00:00.000", TUC);
        
        final double sma = REQ + 250.e3;
        final double ecc = 0.;
        final double inc = FastMath.toRadians(51.6);
        final double raan = FastMath.toRadians(0.);
        final double pa  = FastMath.toRadians(0.);
        final double ano = FastMath.toRadians(0.);
        final KeplerianOrbit iniOrbit = new KeplerianOrbit(sma, ecc, inc, raan, pa, ano, PositionAngle.MEAN, GCRF, date, MU);
        
        // VEHICLE
        
        // Dry mass
        final double dryMass = 1000.;
        final MassProperty dryMassProperty = new MassProperty(dryMass);

        VehicleSurfaceModel vehicleRefSurface = new VehicleSurfaceModel(new Sphere(0.));

        final Vehicle vehicle = new Vehicle(vehicleRefSurface, null, dryMassProperty, null, null, null, null);

        final Assembly assembly = vehicle.createAssembly(GCRF);
        final MassProvider mm = new MassModel(assembly);
        
        // FORCES
        
        // Potential
        final ForceModel potential = EarthGravitationalModelFactory.getDroziner(GravityFieldNames.GRGS, "grim4s4_gr", 8, 8, true);
        
        final ForceModelsData forces = new ForceModelsData(potential, null, null, null, null, null, null, null, null, null, null, null, null,  null, null);
      
        // ATTITUDE
        
        final AttitudeLaw attitudeLaw = new LofOffset(LOFType.TNW);
        
        // INTEGRATION DATA

        final double duration = 3*86400.;
        final double hStop = 0.e+3;
        final PsimuPropagationData propagationData = new PsimuPropagationData(duration, GCRF, hStop);
        
        final double step = 5.;
        final RK4data integrationData = new RK4data(propagationData, step);
        
        System.out.println("Initial orbit:");
        System.out.println("Date:"+iniOrbit.getDate());
        System.out.println("SMA:"+iniOrbit.getA()/1000.+" km");
       
        double dgaRef = 0.;
        double deltaDga = 0.;
        
        for (int i = 0; i < 6; i++) {
            
            // PSIMU INITIALIZATION
            
            Psimu test = null;
            
            switch (i) {
            case 0:
                System.out.println("\nDefault:");
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            case 1:
                System.out.println("\nFactory:");
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.FACTORY, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            case 2:
                System.out.println("\nOnly precession and nutation (manual)");
                FramesFactory.setConfiguration(Utils.getSimplifiedConfiguration(true));
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            case 3:
                System.out.println("\nOnly precession and nutation:");
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            case 4:
                System.out.println("\nEmpty (manual)");
                FramesFactory.setConfiguration(Utils.getSimplifiedConfiguration(false));
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            default:
                System.out.println("\nEmpty:");
                test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.EMPTY, integrationData, vehicle, mm, forces, null, attitudeLaw);
                break;
            }
                     
            
            // PROPAGATION
            
            long debut = System.currentTimeMillis();
            final SpacecraftState sc = test.propagateInSlaveMode();
           
            System.out.println("SMA:"+sc.getA()/1000.+" km");
            //System.out.println("Time:"+(System.currentTimeMillis()-debut)+" ms");
            if ( i == 0 ) {
                dgaRef = sc.getA();
            } else {
                deltaDga = sc.getA() - dgaRef;
                System.out.println("Delta SMA:"+deltaDga+" m");
            }
            
        }


    }
    
}
