import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import fr.cnes.dynvol.psimu.calc.integrator.PsimuPropagationData;
import fr.cnes.dynvol.psimu.calc.integrator.PsimuPropagationData.AltitudeType;
import fr.cnes.dynvol.psimu.calc.integrator.firstOrderIntegrators.RK4data;
import fr.cnes.dynvol.psimu.calc.main.Psimu;
import fr.cnes.dynvol.psimu.calc.main.Psimu.StorageType;
import fr.cnes.dynvol.psimu.calc.main.UserFramesConfsEnum;
import fr.cnes.dynvol.psimu.calc.output.OutputConfig;
import fr.cnes.dynvol.psimu.calc.utils.PsimuUtils;
import fr.cnes.dynvol.psimu.exception.PsimuException;
import fr.cnes.dynvol.psimu.exception.PsimuOutputManagerException;
import fr.cnes.dynvol.psimu.exception.PsimuPropagationException;
import fr.cnes.genius.exception.GArithmeticException;
import fr.cnes.genius.exception.SqliteException;
import fr.cnes.genopus.exception.GPDateException;
import fr.cnes.genopus.nonWidget.exception.NonWidgetException;
import fr.cnes.sirius.addons.patriusdataset.PatriusDataset;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.Vehicle;
import fr.cnes.sirius.patrius.assembly.models.MassModel;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.vehicle.VehicleSurfaceModel;
import fr.cnes.sirius.patrius.attitudes.AttitudeLaw;
import fr.cnes.sirius.patrius.attitudes.LofOffset;
import fr.cnes.sirius.patrius.bodies.ExtendedOneAxisEllipsoid;
import fr.cnes.sirius.patrius.forces.ForceModel;
import fr.cnes.sirius.patrius.forces.ForceModelsData;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory.GravityFieldNames;
import fr.cnes.sirius.patrius.frames.Frame;
import fr.cnes.sirius.patrius.frames.FramesFactory;
import fr.cnes.sirius.patrius.frames.LOFType;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Sphere;
import fr.cnes.sirius.patrius.math.util.FastMath;
import fr.cnes.sirius.patrius.orbits.KeplerianOrbit;
import fr.cnes.sirius.patrius.orbits.PositionAngle;
import fr.cnes.sirius.patrius.propagation.MassProvider;
import fr.cnes.sirius.patrius.propagation.SpacecraftState;
import fr.cnes.sirius.patrius.time.AbsoluteDate;
import fr.cnes.sirius.patrius.time.TimeScale;
import fr.cnes.sirius.patrius.time.TimeScalesFactory;
import fr.cnes.sirius.patrius.utils.Constants;
import fr.cnes.sirius.patrius.utils.exception.PatriusException;

/**
 * Tutorial to use some PSIMU utilities.
 * @author goesterjf
 *
 */
public class PsimuUsingUtils {

    public static void main(String[] args) throws PatriusException, NonWidgetException, IOException, ParseException, PsimuException, PsimuPropagationException, GPDateException, GArithmeticException, SqliteException, PsimuOutputManagerException {
        
        // Patrius Dataset initialization
        PatriusDataset.addResourcesFromPatriusDataset() ;

        // Recovery of the UTC time scale using a "factory"
        final TimeScale TUC = TimeScalesFactory.getUTC();
        
        // Reference frames
        final Frame GCRF = FramesFactory.getGCRF();
        final Frame ITRF = FramesFactory.getITRF();

        // Constants that will be used later
        final double MU   = Constants.WGS84_EARTH_MU;
        final double REQ  = Constants.WGS84_EARTH_EQUATORIAL_RADIUS;
        final double FLAT = Constants.WGS84_EARTH_FLATTENING;
        final ExtendedOneAxisEllipsoid EARTH = new ExtendedOneAxisEllipsoid(REQ, FLAT, ITRF, "EARTH");
        
        // ORBIT
        
        final AbsoluteDate date = new AbsoluteDate("2010-01-01T12:00:00.000", TUC);
        
        final double sma = REQ + 250.e3;
        final double ecc = 0.;
        final double inc = FastMath.toRadians(51.6);
        final double raan = FastMath.toRadians(0.);
        final double pa  = FastMath.toRadians(0.);
        final double ano = FastMath.toRadians(0.);
        final KeplerianOrbit iniOrbit = new KeplerianOrbit(sma, ecc, inc, raan, pa, ano, PositionAngle.MEAN, GCRF, date, MU);
        
        // VEHICLE
        
        // Dry mass
        final double dryMass = 1000.;
        final MassProperty dryMassProperty = new MassProperty(dryMass);
        
        VehicleSurfaceModel vehicleRefSurface = new VehicleSurfaceModel(new Sphere(0.));

        final Vehicle vehicle = new Vehicle(vehicleRefSurface, null, dryMassProperty, null, null, null, null);

        final Assembly assembly = vehicle.createAssembly(GCRF);
        final MassProvider mm = new MassModel(assembly);
        
        // FORCES
        
        // Potential
        final ForceModel potential = EarthGravitationalModelFactory.getDroziner(GravityFieldNames.GRGS, "grim4s4_gr", 8, 8, true);
        
        final ForceModelsData forces = new ForceModelsData(potential, null, null, null, null, null, null, null, null, null, null, null, null,  null, null);
        
        // ATTITUDE
        
        final AttitudeLaw attitudeLaw = new LofOffset(LOFType.TNW);

        // INTEGRATION DATA

        final double duration = -86400.; // Warning => retropolation !!!
        final double hStop = 0.e+3;
        final PsimuPropagationData propagationData = new PsimuPropagationData(duration, GCRF, hStop, AltitudeType.GEODETIC);
        
        final double step = 5.;
        final RK4data integrationData = new RK4data(propagationData, step);

        final StorageType memStore  = StorageType.MEMORY_ALL;
        final StorageType fileStore = StorageType.FILE_ALL;
        
        // PSIMU initialization
        final Psimu test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
        final double outputStep = 60.;
        final OutputConfig output = new OutputConfig(FramesFactory.getEME2000(), LOFType.LVLH, outputStep);   

        test.propagateInMasterMode(output, null, memStore, fileStore);
        test.closeDataBaseConnection();
        List<SpacecraftState> list = test.getSpacecraftStateList();
        final int nb = list.size();
        
        System.out.println();
        System.out.println("Amount of S/C states: "+list.size());
        System.out.println("First element: "+list.get(0).getDate() + "(" + test.getSpacecraftState(0).getDate() + ")");
        System.out.println("Last element:  "+list.get(nb-1).getDate() + "(" + test.getSpacecraftState(nb-1).getDate() + ")");
        
        // Getting the S/C states list from the db file
        
        final List<SpacecraftState> listSc = PsimuUtils.readSpacecraftStatesFromFile(true, true, null, PsimuUtils.BINARY_RESULTS_FILE_NAME, PsimuUtils.BINARY_SC_COLUMN_NAME);
        
        System.out.println();
        System.out.println("Amount of S/C states: "+listSc.size());
        System.out.println("First element: "+listSc.get(0).getDate());
        System.out.println("Last element:  "+listSc.get(listSc.size()-1).getDate());
        
        // Getting a list of variables from the db file

        final ArrayList<String> listOfVarnames =  new ArrayList<String>();
        listOfVarnames.add("DATE");
        listOfVarnames.add("A");
        listOfVarnames.add("E");
        final List<Map<String, Object>> listVar = PsimuUtils.readVariablesFromFile(true, true, null, PsimuUtils.BINARY_RESULTS_FILE_NAME, listOfVarnames);
        
        System.out.println();
        System.out.println("First element:");
        final Map<String, Object> mapi = listVar.get(0);
        for (Map.Entry<String, Object> mapentry : mapi.entrySet()) {
           System.out.println(mapentry.getKey() + " => " + mapentry.getValue());
        }
        System.out.println("Last element:");
        final Map<String, Object> mapf = listVar.get(listVar.size()-1);
        for (Map.Entry<String, Object> mapentry : mapf.entrySet()) {
           System.out.println(mapentry.getKey() + " => " + mapentry.getValue());
        }
        
        // Using S/C state <==> String transformations
        System.out.println();
        final SpacecraftState sc = listSc.get(0);
        final String scString = PsimuUtils.toBase64(sc);
//        System.out.println(scString);
        final SpacecraftState sc2 = PsimuUtils.fromBase64(scString);
        System.out.println("Date: "+sc2.getDate());
        
        // Events names
        
        // Directly from the PSIMU object
        System.out.println();
        final List<Map<String, Object>> listVarEvents = test.getVariablesOnlyEventsList();
        for (Map<String, Object> map : listVarEvents) {
            System.out.println(map.get("EVENT_NAME"));            
        }
        
        // After reading the db file
        System.out.println();
        final ArrayList<String> listOfVarEventsnames =  new ArrayList<String>();
        listOfVarEventsnames.add("EVENT");
        final List<Map<String, Object>> listVarEventsFromFile = PsimuUtils.readVariablesFromFile(false, true, null, PsimuUtils.BINARY_RESULTS_FILE_NAME, listOfVarEventsnames);
        for (Map<String, Object> map : listVarEventsFromFile) {
            for (Map.Entry<String, Object> mapentry : map.entrySet()) {
                System.out.println(mapentry.getValue());
             }
        }

    }
    
    
        
}
